function [ev,grossError,recPM,recPerm] = gerror_sparse(N,n,clstack,truth)
% function [ev,grossError,recPM,recPerm] = gerror_sparse(N,n,clstack,truth)
% Author - Deepti Pachauri 05/31/2013
% Given relative permutation information, gerror_sparse synchronize this information
% and recover the individual permutation for each instance in the dataset. The function establish
% objective matrix using matlab sparse matrix notion. Singular value decomposition is 
% performed using svds.
% Input:
% N - size of the dataset
% n - number of interest points in each instance
% clstack - collection of approximate relative permutations 
% truth - an array of true permutations. This information is only required to estimate errors
% made by the proposed method. If this information is not available then set it to identity for 
% each instance. Method will provide the synchronized permutation but grossError will be 
% meaningless.  
% Output:
% ev - top singular values of the objective matrix. We are interesetd in top n values.
% grossError - error computed using groud truth permutations. 
% recPM - recovered permutation matrices. Each of size nxn
% recPerm - recovered permutation for each instance in dataset. 
%--------------------------------------------------------------

I = speye(n); % sparse identity matrix

% setting up objective matrix using clstack information
clrow = [];
for i = 1:N
    for j = 1:N
        foo = I(:,clstack{i}{j});
        clrow = [clrow;foo];
    end
end


S = []; %objective matrix
ss = [];
for j = 1:N*n:N*N*n
    for i=1:n:n*N
        c = clrow(j+[i-1:i+n-2],:);
        ss = [ss c];
    end
    S = [S;ss];
    ss = [];
    % size(S), pause
end
display('done creating objective matrix...gerror!!!!')
% Sparse singular value decomposition 
[u,s,v] = svds(S,n+10);
ev = diag(s);
% figure, plot(ev,'+')
display('svd S')
% Collecting top-n left singular vectors 
u = u(:,1:n);
% Setting up Permutations for each instance in dataset
B={};
for i = 1:n:N*n
    B{(i+(n-1))/n} = u(i:(i+n)-1,:);
end
recPM={};
for i = 1:N
    tmp =  B{i}*B{1}';
    [tmpU,tmpS,tmpV]=svd(tmp);
    recPM{i} = tmpU*tmpV';
end
recPerm = [];
for i = 1:N
    [c,cost] = munkres(-recPM{i}');
    recPerm = [recPerm; c];
    clear c
end
%-----------------------------------------------
% Error calculation - only correct if truth is provide
grossError = zeros(size(truth));
ind = find(truth-recPerm);
grossError(ind)=1;
grossError = sum(sum(grossError))/(N*n);
