function noise_pm_exp(N,n,noiseX,run)
% function noise_pm_exp(N,n,noiseX,run)
% Author - Deepti Pachauri 05/31/2013
% For a given N, n and noise level (noiseX) this function generate 
% synthetic data and perform permutation synchronization. 
% This function is same as noise_pm_exp_sparse.m 
% except the sparsity part. Dense matrix make this function slow. 
% To generate histograms of singular values use this function.
% Input: 
% N - size of the dataset
% n - number of interest points in each instance
% noiseX - level of noise by which a correct relative permutation
% is switched with a random permutation on Sn
% run - an integer to determine the filename
% Outputs are saved in a mat file.
%-----------------------------------------


%generate N ground truth permutations
truth = zeros(N,n);
truth(1,:) = 1:n;% setting first instance as identity. 
for i = 2:N
    truth(i,:) = randperm(n);
end

% True relative permutations 
I = eye(n);
base = 1:n;
clstack = {}; %collection of true relative permutations

for i = 1:N
    for j = i:N
        a = I(truth(j,:),:);
        c = I(:,truth(i,:))*a;
        c = munkres(-c');
        clstack{i}{j} = c;
        [garbage,foo]=sort(c);
        clstack{j}{i}=base(foo);
        clear foo

    end
end
clear I

% Introducing random permutations 
clstack_random = clstack;

for i = 1:N
    for j = i+1:N
        r = rand(1);
        if r < noiseX
            tt = randperm(n);
            clstack_random{i}{j} = tt;
            [garbage,foo]=sort(tt);
            clstack_random{j}{i}=base(foo);
            clear foo r
        end
    end
end
display('relative assignments comupted')
% display('synchronization on correct data')
% Run this part just to verify the correctness of the synchronization code gerror.
% [STruth,sTruth,gTruth,recPMTruth,recPermTruth] = gerror(N,n,clstack,truth);


display('synchronization on noisy data')
% gerror is the main function. Permutation synchronization for noisy data. 
[SN,snoise,gRandom,recPM,recPerm] = gerror(N,n,clstack_random,truth);

str = strcat('N',num2str(N),'noisePercent',num2str(noiseX*100),'n',num2str(n),'run',num2str(run),'.mat');
save(str)





