function noise_pm_exp_sparse(N,n,noiseX,run)
% function noise_pm_exp_sparse(N,n,noiseX,run)
% Author - Deepti Pachauri 05/31/2013
% For a given N, n and noise level (noiseX) this function generate 
% synthetic data and perform permutation synchronization. 
% After setting the dataset, it calls the main function. That function set up
% objective matrix using matlab's sparse matrix representation hence "sparse" in 
% the function name noise_pm_exp_sparse. This function is same as noise_pm_exp.m 
% except the sparsity part. The synchronization function that noise_pm_exp.m
% calls set up full objective matrix and hence it is slow. Current function 
% do not allow singular value histogram due to the same reason.
% Input: 
% N - size of the dataset
% n - number of interest points in each instance
% noiseX - level of noise by which a correct relative permutation
% is switched with a random permutation on Sn
% run - an integer to determine the filename
% Outputs are saved in a mat file.
%-----------------------------------------


% generate N ground truth permutations
truth = zeros(N,n);
truth(1,:) = 1:n; % setting first instance as identity. 
for i = 2:N
    truth(i,:) = randperm(n);
end

% True relative permutations 
I = eye(n);
base = 1:n;
clstack = {}; %collection of true relative permutations

for i = 1:N
    for j = i:N
        a = I(truth(j,:),:);
        c = I(:,truth(i,:))*a;
        c = munkres(-c');
        clstack{i}{j} = c;
        [garbage,foo]=sort(c);
        clstack{j}{i}=base(foo);
        clear foo c
    end
end

clear I

% Introducing random permutations 
clstack_random = clstack;

for i = 1:N
    for j = i+1:N
        r = rand(1);
        if r < noiseX
            tt = randperm(n);
            clstack_random{i}{j} = tt;
            [garbage,foo]=sort(tt);
            clstack_random{j}{i}=base(foo);
            clear foo r
        end
    end
end
display('relative assignments comupted')

% display('synchronization on correct data')
% Run this part just to verify the correctness of the synchronization code gerror_sparse.
% [s,gTruth, recPMTruth, recPermTruth] = gerror_sparse(N,n,clstack,truth);


display('synchronization on noisy data')
% gerror_sparse is the main function. Permutation synchronization for noisy data. 
[snoise,gRandom,recPM,recPerm] = gerror_sparse(N,n,clstack_random,truth);

str = strcat('sparseN',num2str(N),'noisePercent',num2str(noiseX*100),'n',num2str(n),'run',num2str(run),'.mat');

save(str)






